local M = {}

--Save File: {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15} - Theme 1-3
--           {16, 17, 18, 19} - Trophy Curling, Cue, Target, Airborne
--			 {1} - Darts
--           {15} - Singularity

local function updateTrophies(digit, rangeLow, diamondCount)
	if diamondCount == 15 then
		TorqueScript.eval([[
			trophy_diamond_empty.position = "100000 0 0";
			trophy_diamond.position = "-695.911987 -1540.14001 3";
		]])
	end
	if digit == nil then
		--Forest
		if rangeLow == 0 then 
			TorqueScript.eval([[
				trophy_forest_empty.position = "100000 0 0";
				trophy_forest.position = "-695.911987 -1260.14001 3";
			]])
		--Tech
		elseif rangeLow == 6 then 
			TorqueScript.eval([[
				trophy_tech_empty.position = "100000 0 0";
				trophy_tech.position = "-695.911987 -1330.14001 3";
			]])
		--Space
		elseif rangeLow == 11 then 
			TorqueScript.eval([[
				trophy_space_empty.position = "100000 0 0";
				trophy_space.position = "-695.911987 -1400.14001 3";
			]])
		end
	else
		--Darts
		if digit == 1 then
			local emptyPos, trophyPos = "100000 0 0", "-735.906982 -1641.93994 3"
			TorqueScript.eval([[
				trophy_darts_empty.position = "]]..emptyPos..[[";
				trophy_darts.position = "]]..trophyPos..[[";
			]])
		--Curling
		elseif digit == 16 then 
			local emptyPos, trophyPos = "100000 0 0", "-805.906982 -1641.93994 3"
			TorqueScript.eval([[
				trophy_curling_empty.position = "]]..emptyPos..[[";
				trophy_curling.position = "]]..trophyPos..[[";
			]])
		--Cuegames
		elseif digit == 17 then 
			local emptyPos, trophyPos = "100000 0 0", "-875.906982 -1641.93994 3"
			TorqueScript.eval([[
				trophy_snooker_empty.position = "]]..emptyPos..[[";
				trophy_snooker.position = "]]..trophyPos..[[";
			]])
		--Targets
		elseif digit == 18 then 
			local emptyPos, trophyPos = "100000 0 0", "-945.906982 -1641.93994 3"
			TorqueScript.eval([[
				trophy_targets_empty.position = "]]..emptyPos..[[";
				trophy_targets.position = "]]..trophyPos..[[";
			]])
		--Airborne
		elseif digit == 19 then 
			local emptyPos, trophyPos = "100000 0 0", "-1015.90997 -1641.93994 3"
			TorqueScript.eval([[
				trophy_airborne_empty.position = "]]..emptyPos..[[";
				trophy_airborne.position = "]]..trophyPos..[[";
			]])
			
		--Singularity
		elseif digit == 15 then 
			local emptyPos, trophyPos = "100000 0 0", "-695.911987 -1470.14001 3";
			TorqueScript.eval([[
				trophy_singularity_empty.position = "]]..emptyPos..[[";
				trophy_singularity.position = "]]..trophyPos..[[";
			]])
		end
	end
end

local function fileCheck_trophy(rangeL, rangeU, digit, requirement)
	local file = io.open("cargames_save.txt", "r")
	local medals = file:read()
	file:close()
	
	local update = true
	local diamond = 0
	--loop through save 'number'
	for i = 1, #medals do
		local number = tonumber(medals:sub(i,i))
		if number >= 5 then diamond = diamond+1 end
		
		if (rangeL ~= nil and rangeU ~= nil) and (i > rangeL and i <= rangeU) then --Check range of digit in 'number'
			--Do NOT Update THEME trophy if not >= bronze medal
			if number == 0 then 
				update = false
				break 
			end
		end
		
		if (digit ~= nil and requirement ~= nil) and i == digit then --Check range of digit in 'number'
			--Do NOT Update EVENT trophy if not correct value in 'number'
			if number ~= requirement then update = false end
			break
		end
	end
	
	if update then updateTrophies(digit, rangeL, diamond) end
end

local function getMedal(num)
	if     num == 1             then return 'bronze'
	elseif num == 2             then return 'silver'
	elseif num == 3             then return 'gold'
	elseif num == 4             then return 'platinum'
	elseif num == 5 or num == 6 then return 'diamond'
	end
end

local function fileCheck_medal()
	local file = io.open("cargames_save.txt", "r")
	local medals = file:read()
	file:close()
	
	for i = 1, 15 do --15 being event limit (currently)
		local number = tonumber(medals:sub(i,i))
		if number ~= 0 then --If medal is achieved: ie > 0
			local xCoord = 0
			if     i <= 5  then xCoord = -1138.48+(28*(i-1 ))
			elseif i <= 10 then xCoord = -988.462+(28*(i-6 ))
			elseif i <= 15 then xCoord = -838.487+(28*(i-11)) 
			end
			
			local numberAsMedal = getMedal(number) --Gets medal from number
			TorqueScript.eval(''..numberAsMedal..''..i..'.position = "'..xCoord..' -1236.83997 12.8029003";')
		end
	end
end

local function checkTrophies()
	--Theme Trophies
	fileCheck_trophy(0 ,  5, nil, nil) --Theme 1
	fileCheck_trophy(6 , 10, nil, nil) --Theme 2
	fileCheck_trophy(11, 15, nil, nil) --Theme 3
	
	--Event Trophies
	fileCheck_trophy(nil, nil, 1 , 6) --Darts
	fileCheck_trophy(nil, nil, 16, 1) --Curling
	fileCheck_trophy(nil, nil, 17, 1) --CueGames
	fileCheck_trophy(nil, nil, 18, 1) --Targets
	fileCheck_trophy(nil, nil, 19, 1) --Airborne
	
	--Misc
	fileCheck_trophy(nil, nil, 15, 6) --Singularity
end

local function checkMedals()
	--Checks all medals
	fileCheck_medal()
end

local function onRaceStart()
	checkTrophies()
	checkMedals()
end

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		checkTrophies()
		checkMedals()
	end
end

M.onRaceStart = onRaceStart
M.onScenarioChange = onScenarioChange
return M