--Best = 2250

local M = {}

local helper = require('scenario/scenariohelper')
local scenarioTimeOut = 0
local start = false
local reset = true
local finalScore = 0
local coil = {1, 21, 43, 65}
local ballsPotted = {}
local file = require('scenario/fileHandler')

local function onRaceStart()
	if not start then 
		start = true 
		TorqueScript.eval([[
			triangle_1.position = "10000 0 0";
			triangle_2.position = "10000 0 0";
			triangle_3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
	end
end

local function success(score)
	local medal, nextMedal

	if score < 300 then 
		medal = 'None'
		nextMedal = 'Bronze(300)'
	elseif score >= 300 and score < 500 then 
		medal = 'Bronze'
		nextMedal = 'Silver(500)'
	elseif score >= 500 and score < 700 then 
		medal = 'Silver'
		nextMedal = 'Gold(700)'
	elseif score >= 700 and score < 1000 then 
		medal = 'Gold'
		nextMedal = 'Platinum(1000)'
	elseif score >= 1000 and score < 1350 then 
		medal = 'Platinum'
		nextMedal = 'Diamond(1350)'
	elseif score >= 1350 then 
		medal = 'Diamond'
		nextMedal = 'n/a'
	else
		medal = 'Broken'
		nextMedal = 'Also Broken'
	end
	
	file.updateFile(file.getMedalAsNumber(medal), 8, nil, nil)
	
    local result = {msg = 'Well Done! Your Score: '..score..' | You Achieved: '..medal..' | Next Medal: '..nextMedal..''}
	scenario_scenarios.finish(result)
end

local function fail(reason)
  scenario_scenarios.finish({failed = reason})
end

local function onBeamNGTrigger(data)
	if data.subjectName == 'player' then fail('Foul: You drove into a pocket!')
	elseif (data.triggerName == 'pocket1' or data.triggerName == 'pocket2' or data.triggerName == 'pocket3' or data.triggerName == 'pocket4' or data.triggerName == 'pocket5' or data.triggerName == 'pocket6') and data.subjectName ~= 'player' and data.event == 'enter' then 
		if string.find(data.subjectName, "red") then
			finalScore = finalScore+100
			table.insert(ballsPotted, data.subjectName)
		elseif data.subjectName == 'Black' then
			--Indicates that five balls have been potted that have to be red (else it would have failed already)
			if finalScore >= 450 then
				finalScore = finalScore+200
				finalScore = finalScore+math.floor(80-scenarioTimeOut)*50
				success(finalScore)
			else fail('Foul: The black ball must be potted last!')
			end
		else fail('Foul: A yellow ball was potted! ')
		end
	elseif data.triggerName == 'outside' then
		local potted = false
		
		--Check to see if already potted to stop the scenario failing when ball is potted already
		for key, value in ipairs(ballsPotted) do
			if value == data.subjectName then potted = true end 
		end
		
		if not potted then fail('Foul: A ball flew off the table!') end
	end
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			triangle_1.position = "-563.864563 -1213.02490 29.0229187";
			triangle_2.position = "-555.798157 -1213.27441 29.0229187";
			triangle_3.position = "-559.747009 -1206.63586 29.0327263";
		]])
		be:reloadStaticCollision();
		reset = false
	end
	
	for i = 1, 4 do
		if coil[i] == 70 then coil[i] = 0
		else coil[i] = coil[i]+0.5
		end
	end
	
	TorqueScript.eval([[
		coil1.position = "-654.871582 -1344.68835 ]]..coil[1]..[[";
		coil2.position = "-654.820068 -1157.24207 ]]..coil[2]..[[";
		coil3.position = "-482.854065 -1158.45850 ]]..coil[3]..[[";
		coil4.position = "-482.973755 -1344.71521 ]]..coil[4]..[[";
	]])
end

local function ballsLeft()
	local balls = 0
	for _ in pairs(ballsPotted) do balls = balls+1 end
	
	return 6-balls
end

local function onRaceTick(raceTickTime)
	if start then scenarioTimeOut = scenarioTimeOut+raceTickTime end

	if scenarioTimeOut >= 80 then success(finalScore) end
	
	ui_message('Time Left: '..math.floor(80-scenarioTimeOut)..'s', 1, 'A')
	ui_message('Balls Left: '..ballsLeft(), 1, 'B')
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M