--Best = 4100

local M = {}

local helper = require('scenario/scenariohelper')
local scenarioTimeOut = 0
local start = false
local reset = true
local finalScore = 0
local rotation = 0
local ballsPotted = {}
local powerupUsed = false
local file = require('scenario/fileHandler')

local function onRaceStart()
	if not start then 
		start = true 
		TorqueScript.eval([[
			triangle_1.position = "10000 0 0";
			triangle_2.position = "10000 0 0";
			triangle_3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
	end
end

local function success(score)
	local medal, nextMedal
	
	if score < 2500 then 
		medal = 'None'
		nextMedal = 'Bronze(2500)'
	elseif score >= 2500 and score < 3200 then 
		medal = 'Bronze'
		nextMedal = 'Silver(3200)'
	elseif score >= 3200 and score < 3600 then 
		medal = 'Silver'
		nextMedal = 'Gold(3600)'
	elseif score >= 3600 and score < 3800 then 
		medal = 'Gold'
		nextMedal = 'Platinum(3800)'
	elseif score >= 3800 and score < 4000 then 
		medal = 'Platinum'
		nextMedal = 'Diamond(4000)'
	elseif score >= 4000 then 
		medal = 'Diamond'
		nextMedal = 'n/a'
	else
		medal = 'Broken'
		nextMedal = 'Also Broken'
	end
	
	local extra = ''
	if not powerupUsed and score > 2800 then 
		extra = 'New trophy unlocked!' 
		file.updateFile(file.getMedalAsNumber(medal), 13, 17, 1)
	else file.updateFile(file.getMedalAsNumber(medal), 13, nil, nil)	
	end
	
    local result = {msg = 'Well Done! Your Score: '..score..' | You Achieved: '..medal..' | Next Medal: '..nextMedal..' '..extra..''}
	scenario_scenarios.finish(result)
end

local function fail(reason)
	scenario_scenarios.finish({failed = reason})
end

local function onBeamNGTrigger(data)
	if (data.triggerName == 'pocket1' or data.triggerName == 'pocket2' or data.triggerName == 'pocket3' or data.triggerName == 'pocket4' or data.triggerName == 'pocket5' or data.triggerName == 'pocket6') and data.event == 'enter' then 
		if data.subjectName == 'player' then fail('Foul: You drove into a pocket!')
		else
			table.insert(ballsPotted, data.subjectName)
			local number = tonumber(string.match(data.subjectName, "%d+"))
			if number ~= table.maxn(ballsPotted) then fail('Incorrect Ball Potted!') end
			finalScore = finalScore+(100*number)
			
			if string.find(data.subjectName, "black") then
				finalScore = finalScore+math.floor(60-scenarioTimeOut)*50
				success(finalScore)
			end
		end
	elseif data.triggerName == 'outside' then
		local potted = false
		
		for key, value in ipairs(ballsPotted) do
			if value == data.subjectName then potted = true end 
		end
		
		if not potted then fail('Foul: A ball flew off the table!') end
	elseif string.find(data.triggerName, "time") and data.event == 'enter' then	
		powerupUsed = true
		local timeToAdd = 0
		TorqueScript.eval(''..data.triggerName..'.position = "10000 0 0";')
		if data.triggerName == 'time5_1' then 
			TorqueScript.eval('time5_1_board.position = "10000 0 0";')
			timeToAdd = 5
		elseif data.triggerName == 'time5_2' then 
			TorqueScript.eval('time5_2_board.position = "10000 0 0";')
			timeToAdd = 5
		elseif data.triggerName == 'time10'  then 
			TorqueScript.eval('time10_board.position  = "10000 0 0";')
			timeToAdd = 10
	    end
		
		ui_message(timeToAdd..'s added!', 1, 'Z')
		scenarioTimeOut = scenarioTimeOut-timeToAdd
	end
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			triangle_1.position = "-563.864563 -1213.02490 29.0229187";
			triangle_2.position = "-555.798157 -1213.27441 29.0229187";
			triangle_3.position = "-559.747009 -1206.63586 29.0327263";
			time5_1.position = "-553.186401 -1198.63049 31.2825031";
			time5_2.position = "-587.464966 -1237.82239 31.4032707";
			time10.position  = "-555.549988 -1224.16089 34.3675346";
			time5_1_board.position = "-553.192993 -1198.61890 30.9243927";
			time5_2_board.position = "-587.471558 -1237.81079 31.0451603";
			time10_board.position = "-555.548706 -1224.14734 34.0094223";
		]])
		be:reloadStaticCollision();
		reset = false
	end
	
	rotation = rotation+0.33
	TorqueScript.eval('starSystem.rotation = "0 0 -1 '..rotation..'";')
end

local function onRaceTick(raceTickTime)
	if start then scenarioTimeOut = scenarioTimeOut+raceTickTime end

	if scenarioTimeOut >= 60 then success(finalScore) end
	
	ui_message('Time Left: '..math.floor(60-scenarioTimeOut)..'s', 1, 'A')
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M